/*===========================================================================
 *
 * File:	Textutils.H
 * Author:	Dave Humphrey (uesp@sympatico.ca)
 * Created On:	April 12, 2006
 *
 * Description
 *
 *=========================================================================*/
#ifndef __TEXTUTILS_H
#define __TEXTUTILS_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "oberrorhandler.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Definitions
 *
 *=========================================================================*/

	/* Shortcuts to help define a static text replacement array */
  #define BEGIN_TEXTREPLACE(Name) const textreplace_t Name[] = {
  #define ADD_TEXTREPLACE(Variable, Value) { Variable, Value }, 
  #define END_TEXTREPLACE() { 0, NULL } };

	/* Shortcuts to help define a static string-value array */
  #define BEGIN_STRINGVALUE(Name) const stringvalue_t Name[] = {
  #define ADD_STRINGVALUE(Value, String) { Value, String }, 
  #define END_STRINGVALUE() { 0, NULL } };

/*===========================================================================
 *		End of Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/

	/* Structure used when replacing variables in a text string */
  struct textreplace_t {
	const SSCHAR* pVariable;	/* These are references to strings */
	const SSCHAR* pValue;
   };

	/* Holds information on simple string-value association */
  struct stringvalue_t {
  	int	      Value;
	const SSCHAR* pString;
   };


/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Function Definitions
 *
 *=========================================================================*/

  const SSCHAR* FindTextVariable (const SSCHAR* pString, const textreplace_t ReplaceInfo[]);
  
  const SSCHAR* FindStringValue  (const int Value,       const stringvalue_t StringInfo[]);
  int           FindStringValue  (const SSCHAR* pString, const stringvalue_t StringInfo[]);

  bool          FindStringValue  (int& OutputValue, const SSCHAR* pString, const stringvalue_t StringInfo[]);
  bool          FindStringValue  (const SSCHAR** ppString, const int Value, const stringvalue_t StringInfo[]);

  bool ReplaceTextVariables (CSString& DestString, const CSString& SrcString, const textreplace_t ReplaceInfo[]);

	/* Hexadecimal conversion functions */
  char        ConvertToHexChar     (const dword Value);
  const char* ConvertByteToHexChar (const byte  Value);
  byte        ConvertFromHexByte   (const char* pHexByte);
  dword       ConvertFromHexChar   (const char HexChar);

/*===========================================================================
 *		End of Function Definitions
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File Textutils.H
 *=========================================================================*/
