/*===========================================================================
 *
 * File:	Obfixedparser.CPP
 * Author:	Dave Humphrey (uesp@sympatico.ca)
 * Created On:	October 17, 2006
 *
 * Description
 *
 *=========================================================================*/

	/* Include Files */
#include "obfixedparser.h"
#include "../records/obitem1record.h"


/*===========================================================================
 *
 * Class CObFixedParser Constructor
 *
 *=========================================================================*/
CObFixedParser::CObFixedParser() {
  m_pTokens        = NULL;
  m_pTokenTypes    = NULL;
  m_pRecordHandler = NULL;
  m_pScript        = NULL;
  m_CurrentToken   = 0;
  m_OpCount        = 0;

  m_pCurrentFunction    = NULL;
  m_CurrentFuncParam    = 0;
  m_pOutputData         = NULL;
  m_OutputSize          = 0;
  m_AllocatedOutputSize = 0;
  
  ResetTokens();
}
/*===========================================================================
 *		End of Class CObFixedParser Constructor
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void Destroy (void);
 *
 *=========================================================================*/
void CObFixedParser::Destroy (void) {
  m_pTokens      = NULL;
  m_pTokenTypes  = NULL;
  m_CurrentToken = 0;
  m_OpCount      = 0;

  m_pCurrentFunction = NULL;
  m_CurrentFuncParam = 0;

  m_OutputSizeStack.Destroy();
  m_OutputSizeStack2.Destroy();
  m_LocalVariables.Destroy();
  m_References.Destroy();
  m_ExpressionStack.Destroy();
  m_FunctionParams.Destroy();

  DestroyOutputData();

  ResetTokens();
}
/*===========================================================================
 *		End of Class Method CObFixedParser::Destroy()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void ClearParseData (void);
 *
 *=========================================================================*/
void CObFixedParser::ClearParseData (void) {
  DestroyOutputData();

  m_OutputSizeStack.Destroy();
  m_OutputSizeStack2.Destroy();
  m_LocalVariables.Destroy();
  m_References.Destroy();
  m_ExpressionStack.Destroy();

  m_OpCount          = 0;
  m_CurrentToken     = 0;

  m_pCurrentFunction = NULL;
  m_CurrentFuncParam = 0;  
}
/*===========================================================================
 *		End of Class Method CObFixedParser::ClearParseData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void DestroyOutputData (void);
 *
 *=========================================================================*/
void CObFixedParser::DestroyOutputData (void) {

  if (m_pOutputData != NULL) {
    delete[] m_pOutputData;
    m_pOutputData = NULL;
  }

  m_OutputSize          = 0;
  m_AllocatedOutputSize = 0;
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DestroyOutputData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - word AddReference (FormID);
 *
 *=========================================================================*/
word CObFixedParser::AddReference (const obformid_t FormID) {
  obparseref_t* pRef;

	/* Find an existing reference */
  pRef = FindReference(FormID);
  if (pRef != NULL) return (word) (pRef->Index);

	/* Add a new reference */
  pRef = new obparseref_t;
  m_References.Add(pRef);

  pRef->FormID = FormID;
  pRef->Index  = m_pScript->GetNextLocalRefIndex();

  return (pRef->Index);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::AddReference()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - obparseref_t* FindReference (FormID);
 *
 *=========================================================================*/
obparseref_t* CObFixedParser::FindReference (const obformid_t FormID) {
  obparseref_t* pRef;
  dword         Index;

  for (Index = 0; Index < m_References.GetSize(); ++Index) {
    pRef = m_References.GetAt(Index);
    if (pRef->FormID == FormID) return (pRef);
  }

  return (NULL);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::FindReference()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoMainProgram (void);
 *
 *=========================================================================*/
int CObFixedParser::DoMainProgram (void) {
  tokeninstance_t* pToken;
  int              Result;

	/* Skip any leading lines */
  while (LookAhead(m_TokenEndLine)) {
    MatchToken(m_TokenEndLine);
  }

  Result = DoScriptName();
  if (Result < 0) return (Result);

  while (LookAhead(m_TokenEndofProgram) == NULL) {

    if (LookAhead(m_TokenVarShort)) {
      Result = DoAddLocal(m_TokenVarShort);
    }
    else if (LookAhead(m_TokenVarLong)) {
      Result = DoAddLocal(m_TokenVarLong);
    }
    else if (LookAhead(m_TokenVarFloat)) {
      Result = DoAddLocal(m_TokenVarFloat);
    }
    else if (LookAhead(m_TokenVarRef)) {
      Result = DoAddLocal(m_TokenVarRef);
    }
    else if (LookAhead(m_TokenBegin)) {
      Result = DoBegin();
    }
    else if (LookAhead(m_TokenEndLine)) {
      MatchToken(m_TokenEndLine);
    }
    else {
      pToken = GetCurrentToken();
      AddObGeneralError("Unknown script input '%s' (%s) found!", pToken->Value, pToken->pToken->GetName());
      return (OB_PARSE_RESULT_ERROR);
    }
  }

  pToken = MatchToken(m_TokenEndofProgram, "Unexpected end of script found!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

	/* Check output size */
  if (m_OutputSize > 65535) {
    AddObGeneralError("Exceeded maximum compiler script size of 65535 bytes!");
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoMainProgram()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoScriptName (void);
 *
 *=========================================================================*/
int CObFixedParser::DoScriptName (void) {
  tokeninstance_t* pToken;

  pToken = MatchToken(m_TokenScriptname, "Scripts must begin with a 'scriptname' declaration!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  pToken = MatchToken(m_TokenIdentifier, "Missing or invalid scriptname editorid!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  m_Scriptname = pToken->Value;

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after scriptname declaration!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((dword) 0x0000001D);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoScriptName()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoAddLocal (ocalType);
 *
 *=========================================================================*/
int CObFixedParser::DoAddLocal (const int LocalType) {
  tokeninstance_t* pToken;
  obparselocal_t*  pNewLocal;
  int              LocalIndex;

  pToken = MatchToken(LocalType);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

	/* Ignore if already defined */
  LocalIndex = FindLocalVariableIndex(pToken->Value);

  if (LocalIndex >= 0) {
    AddObGeneralError("Local variable '%s' has already been defined!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Reuse an existing local or create a new one */
  LocalIndex = m_pScript->FindVariableIndex(pToken->Value);
  if (LocalIndex <= 0)  LocalIndex = m_pScript->GetNextLocalVarIndex();

	/* Add the local definition */
  pNewLocal = new obparselocal_t;
  m_LocalVariables.Add(pNewLocal);
  pNewLocal->Index   = LocalIndex;
  pNewLocal->Name    = pToken->Value;
  pNewLocal->TokenID = LocalType;
  pNewLocal->Flags   = (LocalType == m_TokenVarShort || LocalType == m_TokenVarLong) ? OBSLSD_FLAG_SHORTLONG : 0;

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after local variable definition!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoAddLocal()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoBegin (void);
 *
 *=========================================================================*/
int CObFixedParser::DoBegin (void) {
  tokeninstance_t* pToken;
  int              Result;

  pToken = MatchToken(m_TokenBegin);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((word) 0x0010);
  
  Result = DoBeginMode();
  if (Result < 0) return (Result);

  Result = DoStatements();
  if (Result < 0) return (Result);
	
  pToken = MatchToken(m_TokenEnd, "Missing 'end' for previous begin block!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((dword) 0x00000011);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after End!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = PopOutputSizeDWord(m_OutputSize, true);
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoBegin()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoBegin (void);
 *
 *=========================================================================*/
int CObFixedParser::DoBeginMode (void) {
  tokeninstance_t*   pToken;
  CObScriptFunction* pFunction;
  obparselength_t*   pBlockLength;
  int                Result;

  pToken = MatchToken(m_TokenIdentifier, "Missing begin block type!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  pFunction = ObGetScriptConsoleFunction(pToken->Value);

  if (pFunction == NULL) {
    AddObGeneralError("Unknown begin block type '%s'!", pToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

  if (!pFunction->IsBeginFunction()) {
    AddObGeneralError("The function '%s' is not a valid begin block type!", pToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

  m_pCurrentFunction = pFunction;
  m_CurrentFuncParam = 0;

  PushOutputSizeWord(m_OutputSize + 2);
  OutputData(pFunction->GetOpCode());
  pBlockLength = PushOutputSizeDWord(m_OutputSize + 4, true);

  Result = DoFunctionParams();
  if (Result < 0) return (Result);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after begin block type!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = PopOutputSizeWord(m_OutputSize);
  if (Result < 0) return (Result);

	/* Adjust the block size */
  pBlockLength->OutputSize = m_OutputSize;
  
  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoBegin()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParams (void);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParams (void) {
  int  Result;

	/* Ignore if no function defined */
  if (m_pCurrentFunction == NULL) return (OB_PARSE_RESULT_ERROR);

	/* Ignore if function has no parameters */
  if (m_pCurrentFunction->GetNumParameters() == 0) return (OB_PARSE_RESULT_OK);

  PushOutputSizeWord(m_CurrentFuncParam);

	/* Input all function parameters */
  while (m_CurrentFuncParam < (int) m_pCurrentFunction->GetNumParameters()) {
    Result = DoFunctionParam();
    if (Result < 0) return (Result);
    if (Result == OB_PARSE_RESULT_ENDFUNCTION) break;

    ++m_CurrentFuncParam;
  }

	/* Check for any missing parameters */
  Result = DoCheckFunctionEnd();
  if (Result < 0) return (Result);

  Result = PopOutputSizeWord(m_CurrentFuncParam);
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParams()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoCheckFunctionEnd (void);
 *
 *=========================================================================*/
int CObFixedParser::DoCheckFunctionEnd (void) {
  obscriptfuncparam_t* pParam;
  dword		       Index;
  int		       Result = OB_PARSE_RESULT_OK;

  for (Index = m_CurrentFuncParam; Index < m_pCurrentFunction->GetNumParameters(); ++Index) {
    pParam = m_pCurrentFunction->GetParameter(m_CurrentFuncParam);
    if (pParam == NULL) continue;

    if (!pParam->IsOptional()) {
      AddObGeneralError("Missing required parameter #%d", Index + 1);
      Result = OB_PARSE_RESULT_ERROR;
    }
  }

  m_FunctionParams.Empty();
  return (Result);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoCheckFunctionEnd()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParam (void);
 * 
 *=========================================================================*/
int CObFixedParser::DoFunctionParam (void) {
  tokeninstance_t*     pToken;
  obscriptfuncparam_t* pParam;

	/* Ensure a valid parameter */
  pParam = m_pCurrentFunction->GetParameter(m_CurrentFuncParam);

  if (pParam == NULL) {
    AddObGeneralError("Function '%s' does not accept parameter #%d!", m_pCurrentFunction->GetName(), m_CurrentFuncParam + 1);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Match optional comma */
  if (LookAhead(m_TokenComma)) MatchToken(m_TokenComma);

  pToken = GetCurrentToken();
  m_FunctionParams.Add(pToken);

  switch (pParam->Type) {
     case OB_FUNCPARAM_STRING:
	return DoFunctionParamString(pParam);
     case OB_FUNCPARAM_INTEGER:
	return DoFunctionParamInt(pParam);
     case OB_FUNCPARAM_FLOAT:	
	return DoFunctionParamFloat(pParam);
     case OB_FUNCPARAM_ACTORVALUE:
	return DoFunctionParamActorValue(pParam);
     case OB_FUNCPARAM_AXIS:
        return DoFunctionParamAxis(pParam);
     case OB_FUNCPARAM_CRIMETYPE:
	return DoFunctionParamCrime(pParam);
     case OB_FUNCPARAM_ANIMATIONGROUP:
        return DoFunctionParamAnimGroup(pParam);
     case OB_FUNCPARAM_QUESTSTAGE:
        return DoFunctionParamQuestStage(pParam);
     case OB_FUNCPARAM_VARIABLENAME:
	return DoFunctionParamVarName(pParam);
     case OB_FUNCPARAM_SEX:
        return DoFunctionParamSex(pParam);
     case OB_FUNCPARAM_FORMTYPE:
	return DoFunctionParamFormType(pParam);
     case OB_FUNCPARAM_OBJECTREF:
	return DoFunctionParamRefRecord(pParam);
     default:	/* All other record types */
	return DoFunctionParamRecordID(pParam);
  }     
  
  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParam()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamCrime (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamCrime (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  dword            CrimeType;
  bool             Result;

  pToken = MatchToken(m_TokenInteger);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = ObFieldConvertDword(pToken->Value, CrimeType);

  if (Result && CrimeType <= 4) {
    OutputData((word) CrimeType);
  }
  else {
    AddObGeneralError("Unknown crime type '%s' found!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamCrime()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamVarName (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamVarName (obscriptfuncparam_t* pParam) {
  tokeninstance_t*  pToken;
  tokeninstance_t*  pRecordToken;
  CObIdRecord*      pRecord;
  CObIdRecord*      pScriptRecord;
  CObScptRecord*    pScript;
  CObSubrecord*     pSubrecord;
  CObSlsdSubrecord* pLocalSubrecord;
  int               FormType;
  bool              Result;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = GetObFormTypeValue(FormType, pToken->Value);

  if (Result) {
    OutputData((word) FormType);
  }
  else {
    AddObGeneralError("Unknown form type '%s' found!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  if (m_FunctionParams.GetSize() <= 1) {
    AddObGeneralError("No previous record to check variable name '%s'!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* TODO: Move some of variable name getting to script or record handler */
  pRecordToken = m_FunctionParams.GetAt(m_FunctionParams.GetSize() - 1);
  pRecord      = m_pRecordHandler->FindEditorID(pRecordToken->Value);

  if (pRecord == NULL) {
    AddObGeneralError("Cannot check variable name '%s' of unknown record '%s'!", pToken->Value.c_str(), pRecordToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  pSubrecord = pRecord->FindSubrecord(OB_NAME_SCRI);

  if (pSubrecord == NULL) {
    AddObGeneralError("Variable name '%s' is not valid as record '%s' does not have a script!", pToken->Value.c_str(), pRecordToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  pScriptRecord = m_pRecordHandler->FindEditorID((char *)pSubrecord->GetData());

  if (pScriptRecord == NULL) {
    AddObGeneralError("Variable name '%s' is not valid as script '%s' was not found!", pToken->Value.c_str(), (char *)pSubrecord->GetData());
    return (OB_PARSE_RESULT_ERROR);
  }

  pScript = ObCastClass(CObScptRecord, pScriptRecord);

  if (pScript == NULL) {
    AddObGeneralError("Variable name '%s' is not valid as record '%s' is not a script!", pToken->Value.c_str(), pScriptRecord->GetEditorID());
    return (OB_PARSE_RESULT_ERROR);
  }

  pLocalSubrecord = pScript->FindVariable(pToken->Value);

  if (pLocalSubrecord == NULL) {
    AddObGeneralError("Script '%s' does not have a local variable named '%s'!", pScriptRecord->GetEditorID(), pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Output the local variable data */
  OutputData((byte) pLocalSubrecord->IsShortLong() ? 0x73 : 0x66);
  OutputData((word) (word) pLocalSubrecord->GetSlsdData().Index);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamVarName()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamQuestStage (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamQuestStage (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  tokeninstance_t* pQuestToken;
  CObIdRecord*     pRecord;
  CObSubrecord*    pSubrecord;
  dword            QuestStage;
  int		   Position;
  bool             Result;

  pToken = MatchToken(m_TokenInteger);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);
 
  Result = ObFieldConvertDword(pToken->Value, QuestStage);

  if (!Result) {
    AddObGeneralError("Invalid quest stage value '%s'!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  if (m_FunctionParams.GetSize() <= 1) {
    AddObGeneralError("No previous quest to check function parameter quest stage value '%s'!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  pQuestToken = m_FunctionParams.GetAt(m_FunctionParams.GetSize() - 1);
  pRecord = m_pRecordHandler->FindEditorID(pQuestToken->Value);

  if (pRecord == NULL) {
    AddObGeneralError("Cannot check stage value '%s' of unknown quest '%s'!", pToken->Value.c_str(), pQuestToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  if (pRecord->GetRecordType() != OB_NAME_QUST) {
    AddObGeneralError("Cannot check stage value '%s' of non-quest record '%s'!", pToken->Value.c_str(), pQuestToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* TODO: Move to quest record eventually */
  pSubrecord = pRecord->FindFirstSubrecord(OB_NAME_INDX, Position);

  while (pSubrecord != NULL) {
    if (*(word*)pSubrecord->GetData() == QuestStage) break;
    pSubrecord = pRecord->FindNextSubrecord(OB_NAME_INDX, Position);
  }

	/* Not found */
  if (pSubrecord == NULL) {
    AddObGeneralError("The value '%s' is not a valid stage of quest '%s'!", pToken->Value.c_str(), pQuestToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  OutputData((byte) 0x6E);
  OutputData((dword) QuestStage);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamQuestStage()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamFormType (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamFormType (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  int              FormType;
  bool             Result;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = GetObFormTypeValue(FormType, pToken->Value);

  if (Result) {
    OutputData((word) FormType);
  }
  else {
    AddObGeneralError("Unknown form type '%s' found!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamFormType()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamAnimGroup (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamAnimGroup (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  int              AnimGroup;
  bool             Result;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = GetObAnimationGroupValue(AnimGroup, pToken->Value);

  if (Result) {
    OutputData((word) AnimGroup);
  }
  else {
    AddObGeneralError("Unknown animation group '%s' found!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamAnimGroup()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamActorValue (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamActorValue (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  int              ActorValue;
  bool             Result;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = GetObScriptActorValueValue(ActorValue, pToken->Value);

  if (Result) {
    OutputData((word) ActorValue);
  }
  else {
    AddObGeneralError("Unknown actor value '%s' found!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamActorValue()
 *=========================================================================*/
 	

/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamSex (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamSex (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  if (pToken->Value.CompareNoCase("male") == 0) {
    OutputData((word) 0);
  }
  else if (pToken->Value.CompareNoCase("female") == 0) {
    OutputData((word) 1);
  }
  else {
    AddObGeneralError("Unknown axis value '%s' found! Expecting male or female.", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamSex()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamAxis (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamAxis (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  if (pToken->Value.CompareNoCase("x") == 0) {
    OutputData('X');
  }
  else if (pToken->Value.CompareNoCase("y") == 0) {
    OutputData('Y');
  }
  else if (pToken->Value.CompareNoCase("z") == 0) {
    OutputData('Z');
  }
  else {
    AddObGeneralError("Unknown axis value '%s' found! Expecting one of X/Y/Z.", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamAxis()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamInt (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamInt (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  int              Value;

  if (LookAhead(m_TokenInteger)) {
    pToken = MatchToken(m_TokenInteger);
    Value  = atoi(pToken->Value);
  }
  else if (LookAhead(m_TokenFloat)) {
    pToken = MatchToken(m_TokenFloat);
    Value  = (int)atof(pToken->Value);
    AddObGeneralError("Warning: Converting float parameter #%d '%s' to integer!", m_CurrentFuncParam + 1, pToken->Value.c_str());
  }
  else if (LookAhead(m_TokenIdentifier)) {
    
  }
  else if (pParam->IsOptional()) {
    return (OB_PARSE_RESULT_ENDFUNCTION);
  }
  else {
    pToken = GetCurrentToken();
    AddObGeneralError("Function parameter #%d '%s' is not an integer!", m_CurrentFuncParam + 1, pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  OutputData((byte)0x6e);
  OutputData(Value);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamInt()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamFloat (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamFloat (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  double           Value;

  if (LookAhead(m_TokenInteger)) {
    pToken = MatchToken(m_TokenInteger);
    Value  = (double) atoi(pToken->Value);
  }
  else if (LookAhead(m_TokenFloat)) {
    pToken = MatchToken(m_TokenInteger);
    Value  = atof(pToken->Value);
  }
  else if (pParam->IsOptional()) {
    return (OB_PARSE_RESULT_ENDFUNCTION);
  }
  else {
    pToken = GetCurrentToken();
    AddObGeneralError("Function parameter #%d '%s' is not a float!", m_CurrentFuncParam + 1, pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  OutputData((byte)0x6e);
  OutputData(Value);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamFloat()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamString (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamString (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;

  if (LookAhead(m_TokenString)) {
    pToken = MatchToken(m_TokenInteger);
  }
  else if (pParam->IsOptional()) {
    return (OB_PARSE_RESULT_ENDFUNCTION);
  }
  else {
    pToken = GetCurrentToken();
    AddObGeneralError("Function parameter #%d '%s' is not a string!", m_CurrentFuncParam + 1, pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

  if (pToken->Value.GetLength() > 65535) {
    AddObGeneralError("Function parameter #%d string exceeds the maximum length of 65535 bytes!", m_CurrentFuncParam + 1);
    return (OB_PARSE_RESULT_ERROR);
  }

  OutputData((word)pToken->Value.GetLength());
  OutputData(pToken->Value.c_str(), pToken->Value.GetLength());

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamString()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamRecordID (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamRecordID (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  CObRecord*       pRecord;
  word		   RefIndex;

  if (LookAhead(m_TokenIdentifier)) {
    pToken = MatchToken(m_TokenIdentifier);
  }
  else {
    pToken = GetCurrentToken();
    AddObGeneralError("Function parameter #%d '%s' is not an identifier!", m_CurrentFuncParam + 1, pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Find the record */
  pRecord = m_pRecordHandler->FindEditorID(pToken->Value);

  if (pRecord == NULL) {
    AddObGeneralError("Unknown record '%s'!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Check the record type */
  if (!pRecord->IsFormType(pParam->Type)) {
    AddObGeneralError("Function parameter '%s' is not a valid %s type!", pToken->Value.c_str(), ObGetScriptFuncParamString(pParam->Type));
    return (OB_PARSE_RESULT_ERROR);
  }

  	/* Add the reference */
  RefIndex = AddReference(pRecord->GetFormID());

	/* Output the reference data */
  OutputData((byte) 0x72);
  OutputData((word) RefIndex);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamRecordID()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionParamRefRecord (pParam);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionParamRefRecord (obscriptfuncparam_t* pParam) {
  tokeninstance_t* pToken;
  CObRecord*       pRecord;
  word		   RefIndex;

  if (LookAhead(m_TokenIdentifier)) {
    pToken = MatchToken(m_TokenIdentifier);
  }
  else {
    pToken = GetCurrentToken();
    AddObGeneralError("Function parameter #%d '%s' is not an identifier!", m_CurrentFuncParam + 1, pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Find the reference record */
  pRecord = FindReference(pToken->Value);
  
  if (pRecord == NULL) {
    AddObGeneralError("Unknown reference record '%s'!", pToken->Value.c_str());
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Check the record type */
  if (!pRecord->IsFormType(pParam->Type)) {
    AddObGeneralError("Function parameter '%s' is not a valid %s type!", pToken->Value.c_str(), ObGetScriptFuncParamString(pParam->Type));
    return (OB_PARSE_RESULT_ERROR);
  }

  	/* Add the reference */
  RefIndex = AddReference(pRecord->GetFormID());

	/* Output the reference data */
  OutputData((byte) 0x72);
  OutputData((word) RefIndex);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionParamRefRecord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoStatements (void);
 *
 *=========================================================================*/
int CObFixedParser::DoStatements (void) {
  int Result;

  do {
    Result = DoStatement();
    if (Result < 0) return (Result);
  } while (Result != OB_PARSE_RESULT_ENDBLOCK); 

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoStatements()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoStatement (void);
 *
 *=========================================================================*/
int CObFixedParser::DoStatement (void) {

  if (LookAhead(m_TokenVarShort)) {
    return DoAddLocal(m_TokenVarShort);
  }
  else if (LookAhead(m_TokenVarLong)) {
    return DoAddLocal(m_TokenVarLong);
  }
  else if (LookAhead(m_TokenVarFloat)) {
    return DoAddLocal(m_TokenVarFloat);
  }
  else if (LookAhead(m_TokenVarRef)) {
    return DoAddLocal(m_TokenVarRef);
  }
  else if (LookAhead(m_TokenEndLine)) {
    MatchToken(m_TokenEndLine);
    return (OB_PARSE_RESULT_OK);
  }
  else if (LookAhead(m_TokenIdentifier)) {
    return DoFunction();
  }
  else if (LookAhead(m_TokenReturn)) {
    return DoReturn();
  }
  else if (LookAhead(m_TokenSet)) {
    return DoSet();
  }
  else if (LookAhead(m_TokenIf)) {
    return DoIf();
  }

  return (OB_PARSE_RESULT_ENDBLOCK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoStatement()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSet (void);
 *
 *=========================================================================*/
int CObFixedParser::DoSet (void) {
  tokeninstance_t* pToken;
  int              Result;

  MatchToken(m_TokenSet);

  OutputData((word) 0x0015);
  PushOutputSizeWord(m_OutputSize + 2);

  pToken = MatchToken(m_TokenIdentifier, "Missing destination variable for set statement!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  if (LookAhead(m_TokenRef)) {
    Result = DoSetRef(pToken);
    if (Result < 0) return (Result);
  }
  else {
    Result = DoSetCall(pToken);
    if (Result < 0) return (Result);
  }

  PopOutputSizeWord(m_OutputSize);
  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSet()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoVariableRef (pRefToken);
 *
 *=========================================================================*/
int CObFixedParser::DoVariableRef (tokeninstance_t* pRefToken) {
  CObRecord*        pRecord;
  CObRecord*        pScptRecord;
  CObItem1Record*   pItemRecord;
  CObSlsdSubrecord* pLocalSubrecord;
  CObScptRecord*    pScript;
  tokeninstance_t*  pToken;
  word		    RefIndex;
  obformid_t	    FormID;

	/* Ensure a valid reference */
	/* TODO: Actually use a reference here if required */
  pRecord = m_pRecordHandler->FindEditorID(pRefToken->Value);

  if (pRecord == NULL) {
    AddObGeneralError("Unknown reference '%s'!", pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Ensure the record can have a script */
  pItemRecord = ObCastClass(CObItem1Record, pRecord);

  if (pItemRecord == NULL) {
    AddObGeneralError("Invalid reference record '%s'! This type of record can not have a script!", pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

 	/* Add the reference */
  RefIndex = AddReference(pRecord->GetFormID());

	/* Output the reference data */
  OutputData((byte) 0x72);
  OutputData((word) RefIndex);

  pToken = MatchToken(m_TokenRef);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  pToken = MatchToken(m_TokenIdentifier, "Missing reference variable after '.'!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

	/* Does the record have a script? */
  FormID = pItemRecord->GetScript();

  if (FormID == OB_FORMID_NULL) {
    AddObGeneralError("Reference record '%s' does not have a script!", pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Find the script */	
  pScptRecord = m_pRecordHandler->FindFormID(FormID);

  if (pScptRecord == NULL) {
    AddObGeneralError("Script 0x%08X used by record '%s' was not found!", FormID, pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Ensure a valid script */  
  pScript = ObCastClass(CObScptRecord, pScptRecord);

  if (pScptRecord == NULL) {
    AddObGeneralError("Script 0x%08X used by record '%s' is not valid!", FormID, pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Find a local variable in the given object */
  pLocalSubrecord = pScript->FindVariable(pToken->Value);

  if (pLocalSubrecord == NULL) {
    AddObGeneralError("Script '%s' does not have a local variable '%s'!", pScript->GetEditorID(), pToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

  if (pLocalSubrecord->GetSlsdData().Index > 65535) {
    AddObGeneralError("Exceeded the maximum number of local variables (65535) in script '%s'!", pScript->GetEditorID());
    return (OB_PARSE_RESULT_ERROR);
  }

  	/* Output the local variable data */
  OutputData((byte) pLocalSubrecord->IsShortLong() ? 0x73 : 0x66);
  OutputData((word) (word) pLocalSubrecord->GetSlsdData().Index);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoVariableRef()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSetRef (pRefToken);
 *
 *=========================================================================*/
int CObFixedParser::DoSetRef (tokeninstance_t* pRefToken) {
  int Result;

  Result = DoVariableRef(pRefToken);
  if (Result < 0) return (Result);

  return DoSetTo();
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSetRef()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoLocalGlobalOutput (pVarToken);
 *
 *=========================================================================*/
int CObFixedParser::DoLocalGlobalOutput (tokeninstance_t* pVarToken) {
  CObRecord*      pRecord;
  obparselocal_t* pLocal;
  word		  GlobalIndex;

	/* Check for a local first */
  pLocal = FindLocalVariable(pVarToken->Value);

  if (pLocal != NULL) {
    
    if (pLocal->TokenID == m_TokenVarShort || pLocal->TokenID == m_TokenVarLong) {
      OutputData((byte)0x73);
    }
    else if (pLocal->TokenID == m_TokenVarFloat || pLocal->TokenID == m_TokenVarRef) {
      OutputData((byte)0x66);
    }
    else {
      AddObGeneralError("Unknown local variable type %d!", pLocal->TokenID);
      return (OB_PARSE_RESULT_ERROR);
    }

    OutputData((word) pLocal->Index);
    return (OB_PARSE_RESULT_OK);
  }

	/* Check for a global variable */  
  pRecord = m_pRecordHandler->FindEditorID(pVarToken->Value);

  if (pRecord == NULL || pRecord->GetRecordType() != OB_NAME_GLOB) {
    AddObGeneralError("Undefined local or global variable '%s'!", pVarToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

  GlobalIndex = AddReference(pRecord->GetFormID());
  OutputData((byte)0x47);
  OutputData((word)GlobalIndex);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoLocalGlobalOutput()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSetCall (pVarToken);
 *
 *=========================================================================*/
int CObFixedParser::DoSetCall (tokeninstance_t* pVarToken) {
  int Result;

  Result = DoLocalGlobalOutput(pVarToken);
  if (Result < 0) return (Result);

  return DoSetTo();
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSetCall()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSetTo (void);
 *
 *=========================================================================*/
int CObFixedParser::DoSetTo (void) {
  tokeninstance_t* pToken;
  int              Result;

  pToken = MatchToken(m_TokenTo, "Missing 'to' after variable in set statement!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  PushOutputSizeWord(m_OutputSize + 2);

  Result = DoSimpleExpression();
  if (Result < 0) return (Result);

  Result = PopOutputSizeWord(m_OutputSize);
  if (Result < 0) return (Result);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after set expression!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSetTo()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoIf (void);
 *
 *=========================================================================*/
int CObFixedParser::DoIf (void) {
  tokeninstance_t* pToken;
  int              Result;

  pToken = MatchToken(m_TokenIf);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((word) 0x0016);
  PushOutputSizeWord(m_OutputSize + 2);
  PushOutputSizeWord(m_OpCount, true);
  PushOutputSizeWord(m_OutputSize + 2);

  Result = DoSimpleExpression();
  if (Result < 0) return (Result);

  Result = PopOutputSizeWord(m_OutputSize);
  if (Result < 0) return (Result);
  Result = PopOutputSizeWord(m_OutputSize);
  if (Result < 0) return (Result);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after if expression!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);
  
  Result = DoStatements();
  if (Result < 0) return (Result);

  while (LookAhead(m_TokenElseif)) {
    Result = DoElseIf();
    if (Result < 0) return (Result);
  }

  if (LookAhead(m_TokenElse)) {
    Result = DoElse();
    if (Result < 0) return (Result);
  }

  pToken = MatchToken(m_TokenEndif, "Missing 'endif'!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((dword) 0x00000019);
  PopOutputSizeWord(m_OpCount, true);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after endif!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoIf()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoElseIf (void);
 *
 *=========================================================================*/
int CObFixedParser::DoElseIf (void) {
  tokeninstance_t* pToken;
  int              Result;

  PopOutputSizeWord(m_OpCount, true);

  pToken = MatchToken(m_TokenElseif);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((word) 0x0018);
  PushOutputSizeWord(m_OutputSize + 2);
  PushOutputSizeWord(m_OpCount, true);
  PushOutputSizeWord(m_OutputSize + 2);
  
  Result = DoSimpleExpression();
  if (Result < 0) return (Result);

  Result = PopOutputSizeWord(m_OutputSize);
  if (Result < 0) return (Result);
  Result = PopOutputSizeWord(m_OutputSize);
  if (Result < 0) return (Result);
  
  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after elseif expression!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = DoStatements();
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoElseIf()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoElse (void);
 *
 *=========================================================================*/
int CObFixedParser::DoElse (void) {
  tokeninstance_t* pToken;
  int              Result;

  PopOutputSizeWord(m_OpCount, true);

  pToken = MatchToken(m_TokenElse);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  OutputData((word) 0x0017);
  OutputData((word) 0x0002);
  PushOutputSizeWord(m_OpCount, true);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after else statement!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  Result = DoStatements();
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoElse()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSimpleExpression (void);
 *
 *=========================================================================*/
int CObFixedParser::DoSimpleExpression (void) {
  tokeninstance_t* pToken;
  int              Result;

  Result = DoFirstSimpleTerm();
  if (Result < 0) return (Result);

  while (LookAhead(m_TokenAddOp) != NULL) {
    pToken = MatchToken(m_TokenAddOp);

    PopExpressions(m_TokenAddOp);
    PushExpression(pToken);

    Result = DoSimpleTerm();
    if (Result < 0) return (Result);
  }

  if (LookAhead(m_TokenRelOp)) {
    pToken = MatchToken(m_TokenRelOp);

    PopExpressions(m_TokenRelOp);
    PushExpression(pToken);

    Result = DoSimpleExpression();
    if (Result < 0) return (Result);
  }
  else if (LookAhead(m_TokenBoolOp)) {
    pToken = MatchToken(m_TokenBoolOp);

    PopExpressions(m_TokenBoolOp);
    PushExpression(pToken);

    Result = DoSimpleExpression();
    if (Result < 0) return (Result);
  }

  PopExpressionsTo(0);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSimpleExpression()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFirstSimpleTerm (void);
 *
 *=========================================================================*/
int CObFixedParser::DoFirstSimpleTerm (void) {
  tokeninstance_t* pToken;
  int              Result;

  Result = DoFirstSimpleFactor();
  if (Result < 0) return (Result);

  while (LookAhead(m_TokenMultOp) != NULL) {
    pToken = MatchToken(m_TokenMultOp);

    PopExpressions(m_TokenMultOp);
    PushExpression(pToken);
	 
    Result = DoFirstSimpleFactor();
    if (Result < 0) return (Result);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFirstSimpleTerm()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSimpleTerm (void);
 *
 *=========================================================================*/
int CObFixedParser::DoSimpleTerm (void) {
  tokeninstance_t* pToken;
  int              Result;

  Result = DoSimpleFactor();
  if (Result < 0) return (Result);

  while (LookAhead(m_TokenMultOp) != NULL) {
    pToken = MatchToken(m_TokenMultOp);

    PopExpressions(m_TokenMultOp);
    PushExpression(pToken);

    Result = DoFirstSimpleFactor();
    if (Result < 0) return (Result);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSimpleTerm()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFirstSimpleFactor (void);
 *
 *=========================================================================*/
int CObFixedParser::DoFirstSimpleFactor (void) {
  tokeninstance_t* pToken = NULL;
  int              Result;

  if (LookAhead(m_TokenAddOp) != NULL) {
    pToken = MatchToken(m_TokenAddOp);
    if (pToken->Value == "-") pToken->Value = "~";
    PushExpression(pToken);
  }

  Result = DoSimpleFactor();
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFirstSimpleFactor()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSimpleFactor (void);
 *
 *=========================================================================*/
int CObFixedParser::DoSimpleFactor (void) {
  tokeninstance_t* pToken;
  int              Result;

  if (LookAhead(m_TokenLBracket) != NULL) {
    pToken = MatchToken(m_TokenLBracket);
    PushExpression(pToken);
		
    Result = DoSimpleExpression();
    if (Result < 0) return (Result);

    pToken = MatchToken(m_TokenRBracket, "Missing a matching ')'!");
    if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

    PopExpressionsTo(m_TokenLBracket);
  }
  else if (LookAhead(m_TokenInteger) != NULL) {
    Result = DoExprInteger();
    if (Result < 0) return (Result);
  }
  else if (LookAhead(m_TokenFloat) != NULL) {
    Result = DoExprFloat();
    if (Result < 0) return (Result);
  }
  else if (LookAhead(m_TokenIdentifier) != NULL) {
    Result = DoSimpleFactorID();
    if (Result < 0) return (Result);
  }
  else if (LookAhead(m_TokenEndLine) != NULL) {
    AddObGeneralError("Missing expression token before end-of-line!");
    return (OB_PARSE_RESULT_ERROR);
  }
  else {
    pToken = GetCurrentToken();
    AddObGeneralError("Missing expression token before '%s'!", pToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSimpleFactor()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoExprInteger (void);
 *
 *=========================================================================*/
int CObFixedParser::DoExprInteger (void) {
  tokeninstance_t* pToken;

  pToken = MatchToken(m_TokenInteger);

  OutputData((byte) ' ');

	/* TODO */
  //if (m_LastSetNegative) OutputData((byte) '-');
                 
  OutputData(pToken->Value, pToken->Value.GetLength());

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoExprInteger()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoExprFloat (void);
 *
 *=========================================================================*/
int CObFixedParser::DoExprFloat (void) {
  tokeninstance_t* pToken;

  pToken = MatchToken(m_TokenFloat);
  
  OutputData((byte) ' ');

	/* TODO */
  //if (m_LastSetNegative) OutputData((byte) '-');
                 
  OutputData(pToken->Value, pToken->Value.GetLength());

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoExprFloat()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoExprFunction (pFuncToken);
 *
 *=========================================================================*/
int CObFixedParser::DoExprFunction (tokeninstance_t* pFuncToken) {
  CObScriptFunction* pFunction;

  pFunction = ObGetScriptFunction(pFuncToken->Value);

  if (pFunction == NULL) {
    AddObGeneralError("Unknown function '%s'!", pFuncToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Output function data */
  OutputData((byte) ' ');
  OutputData((byte) 0x58);
  OutputData((word) pFunction->GetOpCode());
  PushOutputSizeWord(m_OutputSize + 2);
 
  return DoFunctionHelper(pFunction);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoExprFunction()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoExprRef (pRefToken);
 *
 *=========================================================================*/
int CObFixedParser::DoExprRef (tokeninstance_t* pRefToken) {
  tokeninstance_t*   pToken;
  CObRecord*         pRecord;
  CObScriptFunction* pFunction;
  dword		     RefIndex;

  pToken = m_pTokens->GetAt(m_CurrentToken + 1);

  if (pToken == NULL) {
    AddObGeneralError("Missing variable/function identifier after '.' in reference call!");
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Is this a function? */
  pFunction = ObGetScriptFunction(pToken->Value);
  if (pFunction == NULL) return DoVariableRef(pRefToken);
  
	/* Ensure a valid reference */
  pRecord = m_pRecordHandler->FindEditorID(pRefToken->Value);

  if (pRecord == NULL) {
    AddObGeneralError("Unknown reference '%s'!", pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Add the reference */
  RefIndex = AddReference(pRecord->GetFormID());

	/* Output the reference data */
  OutputData((byte) ' ');
  OutputData((byte) 0x72);
  OutputData((word) RefIndex);

	/* Continue with the function call */
  pToken = MatchToken(m_TokenRef);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  pToken = MatchToken(m_TokenIdentifier, "Missing reference function call after '.'!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  return DoFunctionCall(pToken, true);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoExprRef()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoSimpleFactorID (void);
 *
 *=========================================================================*/
int CObFixedParser::DoSimpleFactorID (void) {
  tokeninstance_t*   pToken;
  CObScriptFunction* pFunction;
  int                Result;

  pToken = MatchToken(m_TokenIdentifier);
  if (LookAhead(m_TokenRef)) return DoExprRef(pToken);

  pFunction = ObGetScriptFunction(pToken->Value);
  if (pFunction != NULL) return DoExprFunction(pToken);

  OutputData((byte) ' ');

  Result = DoLocalGlobalOutput(pToken);
  if (Result < 0) return (Result);
    
  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoSimpleFactorID()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoReturn (void);
 *
 *=========================================================================*/
int CObFixedParser::DoReturn (void) {
  tokeninstance_t* pToken;

  MatchToken(m_TokenReturn);

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after return!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  ++m_OpCount;
  OutputData((dword) 0x0000001E);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoReturn()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunction (void);
 *
 *=========================================================================*/
int CObFixedParser::DoFunction (void) {
  tokeninstance_t* pToken;
  int              Result;

  pToken = MatchToken(m_TokenIdentifier);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  if (LookAhead(m_TokenRef)) {
    Result = DoFunctionRef(pToken);
    if (Result < 0) return (Result);
  }
  else {
    Result = DoFunctionCall(pToken);
    if (Result < 0) return (Result);
  }

  pToken = MatchToken(m_TokenEndLine, "Missing end-of-line after function!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunction()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionRef (pRefToken);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionRef (tokeninstance_t* pRefToken) {
  tokeninstance_t* pToken;
  CObRecord*       pRecord;
  dword		   RefIndex;
  
	/* Ensure a valid reference */
	/* TODO: Ensure a valid reference? */
  pRecord = FindReference(pRefToken->Value);
  //pRecord = m_pRecordHandler->FindEditorID(pRefToken->Value);

  if (pRecord == NULL) {
    AddObGeneralError("Unknown reference '%s'!", pRefToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Add the reference */
  RefIndex = AddReference(pRecord->GetFormID());

	/* Output the reference data */
  OutputData((word) 0x001C);
  OutputData((word) RefIndex);

	/* Continue with the function call */
  pToken = MatchToken(m_TokenRef);
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  pToken = MatchToken(m_TokenIdentifier, "Missing reference function call after '.'!");
  if (pToken == NULL) return (OB_PARSE_RESULT_ERROR);

  return DoFunctionCall(pToken, true);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionRef()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionCall (pFuncToken, IsRefCall);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionCall (tokeninstance_t* pFuncToken, const bool IsRefCall) {
  CObScriptFunction* pFunction;

  pFunction = ObGetScriptFunction(pFuncToken->Value);

  if (pFunction == NULL) {
    AddObGeneralError("Unknown function '%s'!", pFuncToken->Value);
    return (OB_PARSE_RESULT_ERROR);
  }

  if (!pFunction->IsUsesReference() && IsRefCall) {
    SystemLog.Printf("Warning: Reference call on a non-reference function '%s'!", pFunction->GetName());
  }

  OutputData(pFunction->GetOpCode());
  PushOutputSizeWord(m_OutputSize + 2);
  
  return DoFunctionHelper(pFunction);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionCall()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionHelper (pFunction);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionHelper (CObScriptFunction* pFunction) {
  int                Result;

	/* Special case functions */
  switch (pFunction->GetOpCode()) {

    case OB_FUNCOPCODE_GETSCRIPTVARIABLE:
	AddObGeneralError("The function 'GetScriptVariable' cannot be used in a script!");
	return (OB_PARSE_RESULT_ERROR);

    case OB_FUNCOPCODE_GETQUESTVARIABLE:
        AddObGeneralError("The function 'GetQuestVariable' cannot be used in a script!");
	return (OB_PARSE_RESULT_ERROR);
  }
  
	/* Normal functions */
  m_pCurrentFunction = pFunction;
  m_CurrentFuncParam = 0;
  m_FunctionParams.Empty();

  switch (pFunction->GetOpCode()) {
    case OB_FUNCOPCODE_MESSAGE:
        Result = DoFunctionMessage(pFunction);
	break;

    case OB_FUNCOPCODE_MESSAGEBOX:
        Result = DoFunctionMessageBox(pFunction);
	break;

    default:
        Result = DoFunctionParams();
	break;
  }

  m_pCurrentFunction = NULL;
  if (Result < 0) return (Result);
 
  PopOutputSizeWord(m_OutputSize);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionHelper()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionMessage (pFunction);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionMessage (CObScriptFunction* pFunction) {
  tokeninstance_t* pMsgToken;
  tokeninstance_t* pToken;
  int		   ParamCount = 0;
  int		   ButtonCount = 0;
  int              Result;

	/* Parse the main message script */
  pMsgToken = MatchToken(m_TokenString, "Missing main string in MessageBox function call!");
  if (pMsgToken == NULL) return (OB_PARSE_RESULT_ERROR);

  if (pMsgToken->Value.GetLength() > 65535) {
    AddObGeneralError("Exceeded maximum string length of 65535 characters in MessageBox string!");
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Output main message box string */
  m_FunctionParams.Add(pMsgToken);
  OutputData((word) 0x0001);
  OutputData((word) pMsgToken->Value.GetLength());
  OutputData(pMsgToken->Value.c_str(), pMsgToken->Value.GetLength());

  PushOutputSizeWord(0);

	/* Parse all messagebox parameters */
  while (LookAhead(m_TokenIdentifier)) {

    if (ParamCount > OB_SCRIPT_MESSAGE_MAXVARS) {
      PopOutputSizeWord(ParamCount);
      AddObGeneralError("Exceeded maximum number of parameters (%d) in a MessageBox call!", OB_SCRIPT_MESSAGE_MAXVARS);
      return (OB_PARSE_RESULT_ERROR);
    }

    pToken = MatchToken(m_TokenIdentifier);
    m_FunctionParams.Add(pToken);
    ++ParamCount;

    Result = DoLocalGlobalOutput(pToken);
    if (Result < 0) return (Result);
  }

  PopOutputSizeWord(ParamCount);
  
	/* Check the messagebox output variables */
  Result = CheckMessageVariables();
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionMessage()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int DoFunctionMessageBox (pFunction);
 *
 *=========================================================================*/
int CObFixedParser::DoFunctionMessageBox (CObScriptFunction* pFunction) {
  tokeninstance_t* pMsgToken;
  tokeninstance_t* pToken;
  int		   ParamCount = 0;
  int		   ButtonCount = 0;
  int              Result;

	/* Parse the main message script */
  pMsgToken = MatchToken(m_TokenString, "Missing main string in MessageBox function call!");
  if (pMsgToken == NULL) return (OB_PARSE_RESULT_ERROR);

  if (pMsgToken->Value.GetLength() > 65535) {
    AddObGeneralError("Exceeded maximum string length of 65535 characters in MessageBox string!");
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Output main message box string */
  m_FunctionParams.Add(pMsgToken);
  OutputData((word) 0x0001);
  OutputData((word) pMsgToken->Value.GetLength());
  OutputData(pMsgToken->Value.c_str(), pMsgToken->Value.GetLength());

  PushOutputSizeWord(0);

	/* Parse all messagebox parameters */
  while (LookAhead(m_TokenIdentifier)) {

    if (ParamCount > OB_SCRIPT_MESSAGE_MAXVARS) {
      PopOutputSizeWord(ParamCount);
      AddObGeneralError("Exceeded maximum number of parameters (%d) in a MessageBox call!", OB_SCRIPT_MESSAGE_MAXVARS);
      return (OB_PARSE_RESULT_ERROR);
    }

    pToken = MatchToken(m_TokenIdentifier);
    m_FunctionParams.Add(pToken);
    ++ParamCount;

    Result = DoLocalGlobalOutput(pToken);
    if (Result < 0) return (Result);
  }

  PopOutputSizeWord(ParamCount);
  PushOutputSizeWord(0);

	/* Parse all messagebox buttons */
  while (LookAhead(m_TokenString)) {

    if (ButtonCount > OB_SCRIPT_MESSAGE_MAXBUTTONS) {
      PopOutputSizeWord(ParamCount);
      AddObGeneralError("Exceeded maximum number of buttons (%d) in a MessageBox call!", OB_SCRIPT_MESSAGE_MAXVARS);
      return (OB_PARSE_RESULT_ERROR);
    }

    pToken = MatchToken(m_TokenString);
    m_FunctionParams.Add(pToken);
    ++ButtonCount;

    OutputData((word) 0x0001);
    OutputData((word) pToken->Value.GetLength());
    OutputData(pToken->Value.c_str(), pToken->Value.GetLength());
  }

  PopOutputSizeWord(ButtonCount);

	/* Check the messagebox output variables */
  Result = CheckMessageVariables();
  if (Result < 0) return (Result);

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::DoFunctionMessageBox()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - CObIdRecord* FindReference (pEditorID);
 *
 *=========================================================================*/
CObIdRecord* CObFixedParser::FindReference (const char* pEditorID) {

	/* Check for the special player reference */
  if (stricmp(pEditorID, "player") == 0) return m_pRecordHandler->GetPlayerReference();

  return m_pRecordHandler->FindEditorID(pEditorID);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::FindReference()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int CheckMessageVariables (void);
 *
 *=========================================================================*/
int CObFixedParser::CheckMessageVariables (void) {
  tokeninstance_t* pMsgToken;
  tokeninstance_t* pToken;
  const char*      pParse;
  dword            ParamIndex;

  pMsgToken = m_FunctionParams.GetAt(0);
  if (pMsgToken == NULL) return (OB_PARSE_RESULT_OK);

  ParamIndex = 1;
  pParse     = strchr(pMsgToken->Value, '%');

  while (pParse != NULL) {

		/* Special case for the percent sign escape sequence */
    if (pParse[1] == '%') {
      ++pParse;
    }
    else {
      pToken = m_FunctionParams.GetAt(ParamIndex);

      if (pToken == NULL) {
        AddObGeneralError("Missing output variable #%d in Message/MessageBox call!", ParamIndex);
      }
      else if (pToken->pToken->GetID() == m_TokenString) {
        AddObGeneralError("Missing output variable #%d in Message/MessageBox call!", ParamIndex);
      }

      ++ParamIndex;  
    }

    pParse = strchr(pParse + 1, '%');
  }

	/* Check for extra variables */
  for (; ParamIndex < m_FunctionParams.GetSize(); ++ParamIndex) {
    pToken = m_FunctionParams.GetAt(ParamIndex);
    if (pToken->pToken->GetID() == m_TokenString) break;

    AddObGeneralError("Extra output variable #%d (%s) in Message/MessageBox call!", ParamIndex, pToken->Value.c_str());
  }

  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::CheckMessageVariables()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int FindLocalVariableIndex (pString);
 *
 *=========================================================================*/
int CObFixedParser::FindLocalVariableIndex (const char* pString) {
  obparselocal_t* pLocal;
  
  pLocal = FindLocalVariable(pString);
  if (pLocal != NULL) return (pLocal->Index);

  return (-1);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::FindLocalVariableIndex()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - obparselocal_t* FindLocalVariable (pString);
 *
 *=========================================================================*/
obparselocal_t* CObFixedParser::FindLocalVariable (const char* pString) {
  obparselocal_t* pLocal;
  dword           Index;

  for (Index = 0; Index < m_LocalVariables.GetSize(); ++Index) {
    pLocal = m_LocalVariables.GetAt(Index);
    if (pLocal->Name.CompareNoCase(pString) == 0) return (pLocal);
  }

  return (NULL);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::FindLocalVariable()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - tokeninstance_t* MatchToken (TokenID, pErrorString);
 *
 * Uses a custom error string if the token does not match.
 *
 *=========================================================================*/
tokeninstance_t* CObFixedParser::MatchToken (const int TokenID, const char* pErrorString) {
  tokeninstance_t* pToken;

  pToken = m_pTokens->GetAt(m_CurrentToken);

  if (pToken == NULL) {
    AddObGeneralError("Unexpected end of program found!");
    return (NULL);
  }
 
  assert(pToken->pToken != NULL);

  if (pToken->pToken->GetID() != TokenID) {
    AddObGeneralError(pErrorString);
    return (NULL);
  }

  ++m_CurrentToken;
  return (pToken);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::MatchToken()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - tokeninstance_t* MatchToken (TokenID);
 *
 * Matches the next token in the stream to the given type. On success the
 * token stream is advanced one and the token is returns. Returns NULL on any
 * error if the next token is not the given type.
 *
 *=========================================================================*/
tokeninstance_t* CObFixedParser::MatchToken (const int TokenID) {
  tokeninstance_t* pToken;

  pToken = m_pTokens->GetAt(m_CurrentToken);

  if (pToken == NULL) {
    AddObGeneralError("Unexpected end of program found!");
    return (NULL);
  }
 
  assert(pToken->pToken != NULL);

  if (pToken->pToken->GetID() != TokenID) {
    CTokenType* pType = m_pTokenTypes->GetTokenType(TokenID);
    AddObGeneralError("Expected '%s' but found '%s'!", pType ? pType->GetName() : "n/a", pToken->pToken->GetName());
    return (NULL);
  }

  ++m_CurrentToken;
  return (pToken);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::MatchToken()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - tokeninstance_t* LookAhead (TokenID);
 *
 * Matches the next token in the stream to the given type. On success the
 * token stream is advanced one and the token is returns. Returns NULL on any
 * error if the next token is not the given type.
 *
 *=========================================================================*/
tokeninstance_t* CObFixedParser::LookAhead (const int TokenID) {
  tokeninstance_t* pToken;

  pToken = m_pTokens->GetAt(m_CurrentToken);
  if (pToken == NULL) return (NULL);
   
  assert(pToken->pToken != NULL);
  if (pToken->pToken->GetID() != TokenID) return (NULL);

  return (pToken);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::LookAhead()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - tokeninstance_t* GetCurrentToken (void);
 *
 *=========================================================================*/
tokeninstance_t* CObFixedParser::GetCurrentToken (void) {
  return m_pTokens->GetAt(m_CurrentToken);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::GetCurrentToken()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - bool OutputData (Value);
 *
 *=========================================================================*/
bool CObFixedParser::OutputData (const char   Value) {  return OutputData((byte*)&Value, 1); }
bool CObFixedParser::OutputData (const byte   Value) {  return OutputData((byte*)&Value, 1); }
bool CObFixedParser::OutputData (const short  Value) {  return OutputData((byte*)&Value, 2); }
bool CObFixedParser::OutputData (const word   Value) {  return OutputData((byte*)&Value, 2); }
bool CObFixedParser::OutputData (const dword  Value) {  return OutputData((byte*)&Value, 4); }
bool CObFixedParser::OutputData (const int    Value) {  return OutputData((byte*)&Value, 4); }
bool CObFixedParser::OutputData (const float  Value) {  return OutputData((byte*)&Value, 4); }
bool CObFixedParser::OutputData (const double Value) {  return OutputData((byte*)&Value, 8); }
/*===========================================================================
 *		End of Class Method CObFixedParser::OutputData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - bool OutputData (pData, Size);
 *
 *=========================================================================*/
bool CObFixedParser::OutputData (const byte* pData, const dword Size) {
  byte* pNewData;
  dword NewSize;
  
	/* Grow array as needed */
  if (m_OutputSize + Size >= m_AllocatedOutputSize) {
    NewSize = m_AllocatedOutputSize*2 + 256;
    pNewData = new byte[NewSize];
    m_AllocatedOutputSize = NewSize;

    if (m_pOutputData != NULL) {
      memcpy(pNewData, m_pOutputData, m_OutputSize);
      delete[] m_pOutputData;
    }

    m_pOutputData = pNewData;
  }

	/* Copy data */
  memcpy(m_pOutputData + m_OutputSize, pData, Size);
  m_OutputSize += Size;

  return (true);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::OutputData()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - bool Parse (void);
 *
 *=========================================================================*/
bool CObFixedParser::Parse (void) {
  int Result;
  
  ClearParseData();
  
  if (m_pTokens == NULL || m_pTokenTypes == NULL) {
    AddObGeneralError("Script token data has not been set!");
    return (false);
  }

  if (m_pRecordHandler == NULL) {
    AddObGeneralError("Script record handler has not been set!");
    return (false);
  }

  if (m_pScript == NULL) {
    AddObGeneralError("Script has not been set!");
    return (false);
  }

  ResolveTokenTypes();

  Result = DoMainProgram();
  if (Result < 0) return (false);

  CObFile File;

  File.Open("test.dat", "wb");
  File.Write(m_pOutputData, m_OutputSize);
  File.Close();

  return (true);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::Parse()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - obparselength_t* PushOutputSizeWord (Size, UseSecondStack);
 *
 *=========================================================================*/
obparselength_t* CObFixedParser::PushOutputSizeWord (const dword Size, const bool UseSecondStack) {
  obparselength_t* pLength;

  pLength = new obparselength_t;

  if (UseSecondStack)
    m_OutputSizeStack2.Add(pLength);
  else
    m_OutputSizeStack.Add(pLength);

  pLength->OutputPos  = m_OutputSize;
  
	/* Add the placeholder size to the output data */
  OutputData((word) 0x0000);

  pLength->OutputSize = Size;

  return (pLength);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::PushOutputSizeWord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int PopOutputSizeWord (Size, UseSecondStack);
 *
 *=========================================================================*/
int CObFixedParser::PopOutputSizeWord (const dword Size, const bool UseSecondStack) {
  obparselength_t*     pLength;
  CObParseLengthStack* pStack = &m_OutputSizeStack;
  dword		       Index;

  if (UseSecondStack) pStack = &m_OutputSizeStack2;

  if (pStack->GetSize() == 0) {
    AddObGeneralError("Popped an empty output size stack (this shouldn't happen)!");
    return (OB_PARSE_RESULT_ERROR);
  }

  Index   = pStack->GetSize() - 1;
  pLength = pStack->GetAt(Index);
  if (pLength == NULL) return (OB_PARSE_RESULT_ERROR);

	/* Sanity check */
  if (Size < pLength->OutputSize) {
    AddObGeneralError("Invalid negative output size length found!");
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Check for size overflows */
  dword DiffSize = (Size - pLength->OutputSize);

  if (DiffSize > 65535) {
    AddObGeneralError("Output size overflow!");
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Update the length in the compiler data */
  *(word *)(m_pOutputData + pLength->OutputPos) = (word) DiffSize;

  pStack->Delete(Index);
  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::PopOutputSizeWord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - obparselength_t* PushOutputSizeDWord (Size, UseSecondStack);
 *
 *=========================================================================*/
obparselength_t* CObFixedParser::PushOutputSizeDWord (const dword Size, const bool UseSecondStack) {
  obparselength_t* pLength;

  pLength = new obparselength_t;

  if (UseSecondStack)
    m_OutputSizeStack2.Add(pLength);
  else
    m_OutputSizeStack.Add(pLength);

  pLength->OutputPos = m_OutputSize;
  
	/* Add the placeholder size to the output data */
  OutputData((dword) 0x00000000);

  pLength->OutputSize = Size;
  return (pLength);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::PushOutputSizeDWord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - int PopOutputSizeDWord (Size, UseSecondStack);
 *
 *=========================================================================*/
int CObFixedParser::PopOutputSizeDWord (const dword Size, const bool UseSecondStack) {
  obparselength_t*     pLength;
  CObParseLengthStack* pStack = &m_OutputSizeStack;
  dword		       Index;

  if (UseSecondStack) pStack = &m_OutputSizeStack2;

  if (pStack->GetSize() == 0) {
    AddObGeneralError("Popped an empty output size stack (this shouldn't happen)!");
    return (OB_PARSE_RESULT_ERROR);
  }

  Index   = pStack->GetSize() - 1;
  pLength = pStack->GetAt(Index);
  if (pLength == NULL) return (OB_PARSE_RESULT_ERROR);

	/* Sanity check */
  if (Size < pLength->OutputSize) {
    AddObGeneralError("Invalid negative output size length found!");
    return (OB_PARSE_RESULT_ERROR);
  }

	/* Check for size overflows */
  dword DiffSize = (Size - pLength->OutputSize);

	/* Update the length in the compiler data */
  *(dword *)(m_pOutputData + pLength->OutputPos) = DiffSize;

  pStack->Delete(Index);
  return (OB_PARSE_RESULT_OK);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::PopOutputSizeDWord()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void PopExpressionsTo (TokenID);
 *
 *=========================================================================*/
void CObFixedParser::PopExpressionsTo (const int TokenID) {
  tokeninstance_t* pToken;
  int		   Index = m_ExpressionStack.GetSize() - 1;

  pToken = m_ExpressionStack.GetAt(Index);
  if (pToken == NULL) return;

  do {
    m_ExpressionStack.Delete(Index);

    if (pToken->pToken->GetID() != m_TokenLBracket) {
      OutputData((byte) ' ');
      OutputData(pToken->Value, pToken->Value.GetLength());
    }

    if (pToken->pToken->GetID() == TokenID) break;

    Index  = m_ExpressionStack.GetSize() - 1;
    pToken = m_ExpressionStack.GetAt(Index);
  } while (pToken != NULL);

}
/*===========================================================================
 *		End of Class Method CObFixedParser::PopExpressionsTo()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void PopExpressions (TokenID);
 *
 *=========================================================================*/
void CObFixedParser::PopExpressions (const int TokenID) {
  tokeninstance_t* pToken;
  bool             Match;
  int		   Index = m_ExpressionStack.GetSize() - 1;

  pToken = m_ExpressionStack.GetAt(Index);
  
  while (pToken != NULL) {
    Match = false;

    if (pToken->pToken->GetID() == TokenID) 
      Match = true;
    else if (TokenID == m_TokenAddOp)
      Match = (pToken->pToken->GetID() == m_TokenMultOp);
    else if (TokenID == m_TokenRelOp)
      Match = (pToken->pToken->GetID() == m_TokenMultOp) || (pToken->pToken->GetID() == m_TokenAddOp);
    else if (TokenID == m_TokenBoolOp)
      Match = (pToken->pToken->GetID() == m_TokenMultOp) || (pToken->pToken->GetID() == m_TokenAddOp) || (pToken->pToken->GetID() == m_TokenRelOp);

    if (!Match) break;

    m_ExpressionStack.Delete(Index);

    if (pToken->pToken->GetID() != m_TokenLBracket) {
      OutputData((byte) ' ');
      OutputData(pToken->Value, pToken->Value.GetLength());
    }
   
    Index  = m_ExpressionStack.GetSize() - 1;
    pToken = m_ExpressionStack.GetAt(Index);
  }

}
/*===========================================================================
 *		End of Class Method CObFixedParser::PopExpressions()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void PushExpression (pToken);
 *
 *=========================================================================*/
void CObFixedParser::PushExpression (tokeninstance_t* pToken) {
  m_ExpressionStack.Add(pToken);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::PushExpression()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - void ResetTokens (void);
 *
 *=========================================================================*/
void CObFixedParser::ResetTokens (void) {
  m_TokenUnknown	= TOKEN_ID_UNKNOWN;
  m_TokenLBracket	= TOKEN_ID_UNKNOWN;
  m_TokenRBracket	= TOKEN_ID_UNKNOWN;
  m_TokenAddOp		= TOKEN_ID_UNKNOWN;
  m_TokenMultOp		= TOKEN_ID_UNKNOWN;
  m_TokenRelOp		= TOKEN_ID_UNKNOWN;
  m_TokenBoolOp		= TOKEN_ID_UNKNOWN;
  m_TokenString		= TOKEN_ID_UNKNOWN;
  m_TokenIdentifier	= TOKEN_ID_UNKNOWN;
  m_TokenIf		= TOKEN_ID_UNKNOWN;
  m_TokenElseif		= TOKEN_ID_UNKNOWN;
  m_TokenElse		= TOKEN_ID_UNKNOWN;
  m_TokenEndif		= TOKEN_ID_UNKNOWN;
  m_TokenSet		= TOKEN_ID_UNKNOWN;
  m_TokenScriptname	= TOKEN_ID_UNKNOWN;
  m_TokenBegin		= TOKEN_ID_UNKNOWN;
  m_TokenEnd		= TOKEN_ID_UNKNOWN;
  m_TokenEndLine	= TOKEN_ID_UNKNOWN;
  m_TokenInteger	= TOKEN_ID_UNKNOWN;
  m_TokenFloat		= TOKEN_ID_UNKNOWN;
  m_TokenRef		= TOKEN_ID_UNKNOWN;
  m_TokenReturn		= TOKEN_ID_UNKNOWN;
  m_TokenComma		= TOKEN_ID_UNKNOWN;
  m_TokenEndofProgram	= TOKEN_ID_UNKNOWN;
  m_TokenVarShort	= TOKEN_ID_UNKNOWN;
  m_TokenVarLong	= TOKEN_ID_UNKNOWN;
  m_TokenVarFloat	= TOKEN_ID_UNKNOWN;
  m_TokenVarRef		= TOKEN_ID_UNKNOWN;
  m_TokenTo		= TOKEN_ID_UNKNOWN;
}
/*===========================================================================
 *		End of Class Method CObFixedParser::ResetTokens()
 *=========================================================================*/


/*===========================================================================
 *
 * Class CObFixedParser Method - bool ResolveTokenTypes (void);
 *
 * Finds the current value of required script tokens. Returns false if any
 * are not found.
 *
 *=========================================================================*/
bool CObFixedParser::ResolveTokenTypes (void) {

	/* Abort if token types are not set */
  if (m_pTokenTypes == NULL) return AddObGeneralError("Cannot resolve token types as array has not been set!");

  m_TokenUnknown	= m_pTokenTypes->FindTokenID("Unknown");
  if ((m_TokenLBracket	= m_pTokenTypes->FindTokenID("LBracket")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenRBracket	= m_pTokenTypes->FindTokenID("RBracket")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenAddOp	= m_pTokenTypes->FindTokenID("Addop")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenMultOp	= m_pTokenTypes->FindTokenID("Multop")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenRelOp	= m_pTokenTypes->FindTokenID("Relop")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenBoolOp	= m_pTokenTypes->FindTokenID("Boolop")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenString	= m_pTokenTypes->FindTokenID("String")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenIdentifier= m_pTokenTypes->FindTokenID("Identifier")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenIf	= m_pTokenTypes->FindTokenID("If")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenElseif	= m_pTokenTypes->FindTokenID("Elseif")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenElse	= m_pTokenTypes->FindTokenID("Else")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenEndif	= m_pTokenTypes->FindTokenID("Endif")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenSet	= m_pTokenTypes->FindTokenID("Set")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenScriptname= m_pTokenTypes->FindTokenID("Scriptname")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenBegin	= m_pTokenTypes->FindTokenID("Begin")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenEnd	= m_pTokenTypes->FindTokenID("End")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenEndLine	= m_pTokenTypes->FindTokenID("Endline")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenInteger	= m_pTokenTypes->FindTokenID("Integer")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenFloat	= m_pTokenTypes->FindTokenID("Float")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenRef	= m_pTokenTypes->FindTokenID("Ref")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenReturn	= m_pTokenTypes->FindTokenID("Return")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenComma	= m_pTokenTypes->FindTokenID("Comma")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenEndofProgram = m_pTokenTypes->FindTokenID("Endofprogram")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenVarShort	= m_pTokenTypes->FindTokenID("VarShort")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenVarLong	= m_pTokenTypes->FindTokenID("VarLong")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenVarFloat	= m_pTokenTypes->FindTokenID("VarFloat")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenTo	= m_pTokenTypes->FindTokenID("To")) == TOKEN_ID_UNKNOWN) return (false);
  if ((m_TokenVarRef	= m_pTokenTypes->FindTokenID("VarRef")) == TOKEN_ID_UNKNOWN) return (false);
  

  return (true);
}
/*===========================================================================
 *		End of Class Method CObFixedParser::ResolveTokenTypes()
 *=========================================================================*/
 