/*===========================================================================
 *
 * File:	ObRefrRecord.H
 * Author:	Dave Humphrey (uesp@sympatico.ca)
 * Created On:	April 12, 2006
 *
 * Defines the CObRefrRecord class for reference records.
 *
 *=========================================================================*/
#ifndef __OBREFRRECORD_H
#define __OBREFRRECORD_H


/*===========================================================================
 *
 * Begin Required Includes
 *
 *=========================================================================*/
  #include "obidrecord.h"
  #include "../subrecords/obdwordsubrecord.h"
  #include "../subrecords/obwordsubrecord.h"
  #include "../subrecords/obrefrdatasubrecord.h"
  #include "../subrecords/obxtelsubrecord.h"
  #include "../subrecords/obformidsubrecord.h"
  #include "../subrecords/obxespsubrecord.h"
  #include "../subrecords/obxlocsubrecord.h"
/*===========================================================================
 *		End of Required Includes
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Type Definitions
 *
 *=========================================================================*/

/*===========================================================================
 *		End of Type Definitions
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Class CObRefrRecord Definition
 *
 * Holds reference data.
 *
 *=========================================================================*/
class CObRefrRecord : public CObIdRecord {
  DECLARE_OBSUBRECCREATE()
  DECLARE_OBFIELDMAP()
  DECLARE_OBCLASS(CObRefrRecord, CObIdRecord)
  
  /*---------- Begin Protected Class Members --------------------*/
protected:
  CObFormidSubrecord*	m_pName;
  CObFullSubrecord*	m_pItemName;
  CObRefrDataSubrecord* m_pData;
  CObXtelSubrecord*	m_pTeleport;
  CObWordSubrecord*	m_pMarkerType;


  /*---------- Begin Protected Class Methods --------------------*/
protected:


  /*---------- Begin Public Class Methods -----------------------*/
public:

	/* Class Constructors/Destructors */
  CObRefrRecord();
  //virtual ~CObRefrRecord() { Destroy(); }
  virtual void Destroy (void);

  	/* Return a new instance of the class */
  static CObRecord* Create (void) { return new CObRefrRecord; }

	/* Get class members */
  const SSCHAR* GetItemName       (void) { return (m_pItemName ? m_pItemName->GetString()    : ""); }
  obrefrdata_t* GetRefrData       (void) { return (m_pData     ? &m_pData->GetRefrData()     : NULL); }
  obxteldata_t* GetTeleportData   (void) { return (m_pTeleport ? &m_pTeleport->GetXtelData() : NULL); }
  float         GetLocationX      (void) { return (m_pData     ? m_pData->GetX()	     : 0); }
  float         GetLocationY      (void) { return (m_pData     ? m_pData->GetY()	     : 0); }
  float         GetLocationZ      (void) { return (m_pData     ? m_pData->GetZ()	     : 0); }
  float         GetAngleX         (void) { return (m_pData     ? m_pData->GetAX()	     : 0); }
  float         GetAngleY         (void) { return (m_pData     ? m_pData->GetAY()	     : 0); }
  float         GetAngleZ         (void) { return (m_pData     ? m_pData->GetAZ()	     : 0); }
  obformid_t    GetTeleportFormID (void) { return (m_pTeleport ? m_pTeleport->GetFormID()    : OB_FORMID_NULL); }
  obformid_t	GetBaseFormID	  (void) { return (m_pName     ? m_pName->GetValue()	     : OB_FORMID_NULL); }
  float         GetTeleportLocX   (void) { return (m_pTeleport ? m_pTeleport->GetX()	     : 0); }
  float         GetTeleportLocY   (void) { return (m_pTeleport ? m_pTeleport->GetY()	     : 0); }
  float         GetTeleportLocZ   (void) { return (m_pTeleport ? m_pTeleport->GetZ()	     : 0); }
  float         GetTeleportAngleX (void) { return (m_pTeleport ? m_pTeleport->GetAX()	     : 0); }
  float         GetTeleportAngleY (void) { return (m_pTeleport ? m_pTeleport->GetAY()	     : 0); }
  float         GetTeleportAngleZ (void) { return (m_pTeleport ? m_pTeleport->GetAZ()	     : 0); }
  word		GetMarkerType     (void) { return (m_pMarkerType ? m_pMarkerType->GetValue() : 0); }
  bool		IsMapMarker	  (void) { return (m_pMarkerType != NULL); }
  bool		IsActor           (void);

	/* Check the form type for the reference */
  virtual bool IsFormType (const int Type);

	/* Used to determine if the record type has a FULL item name field */
  virtual bool HasFullItemName (void) { return (true); }

	/* Initialize a new record */
  void InitializeNew (void);

	/* Called to alert record of a new subrecord being added */
  virtual void OnAddSubrecord    (CObSubrecord* pSubrecord);
  virtual void OnDeleteSubrecord (CObSubrecord* pSubrecord);

	/* Set class members */
  void SetItemName   (const SSCHAR* pString);
  void SetBaseFormID (const obformid_t FormID);

	/* Begin field method definitions */
  DECLARE_OBGETFIELD(GetFieldItemName)
  DECLARE_OBGETFIELD(GetFieldLocationX)
  DECLARE_OBGETFIELD(GetFieldLocationY)
  DECLARE_OBGETFIELD(GetFieldLocationZ)
  DECLARE_OBGETFIELD(GetFieldAngleX)
  DECLARE_OBGETFIELD(GetFieldAngleY)
  DECLARE_OBGETFIELD(GetFieldAngleZ)
  DECLARE_OBGETFIELD(GetFieldTeleportFormID)
  DECLARE_OBGETFIELD(GetFieldTeleportName)
  DECLARE_OBGETFIELD(GetFieldTeleportLocX)
  DECLARE_OBGETFIELD(GetFieldTeleportLocY)
  DECLARE_OBGETFIELD(GetFieldTeleportLocZ)
  DECLARE_OBGETFIELD(GetFieldTeleportAngleX)
  DECLARE_OBGETFIELD(GetFieldTeleportAngleY)
  DECLARE_OBGETFIELD(GetFieldTeleportAngleZ)
  DECLARE_OBGETFIELD(GetFieldWorldSpace)
  DECLARE_OBGETFIELD(GetFieldMarkerType)
  DECLARE_OBGETFIELD(GetFieldMarkerTypeID)
  DECLARE_OBGETFIELD(GetFieldBaseFormID)
  DECLARE_OBGETFIELD(GetFieldBaseEditorID)
  DECLARE_OBGETFIELD(GetFieldBaseName)

  DECLARE_OBCOMPFIELD(CompareFieldItemName)
  DECLARE_OBCOMPFIELD(CompareFieldLocationX)
  DECLARE_OBCOMPFIELD(CompareFieldLocationY)
  DECLARE_OBCOMPFIELD(CompareFieldLocationZ)
  DECLARE_OBCOMPFIELD(CompareFieldAngleX)
  DECLARE_OBCOMPFIELD(CompareFieldAngleY)
  DECLARE_OBCOMPFIELD(CompareFieldAngleZ)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportFormID)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportLocX)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportLocY)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportLocZ)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportAngleX)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportAngleY)
  DECLARE_OBCOMPFIELD(CompareFieldTeleportAngleZ)
  DECLARE_OBCOMPFIELD(CompareFieldMarkerType)
  DECLARE_OBCOMPFIELD(CompareFieldMarkerTypeID)
  DECLARE_OBCOMPFIELD(CompareFieldBaseFormID)
  DECLARE_OBCOMPFIELD(CompareFieldBaseEditorID)

  DECLARE_OBSETFIELD(SetFieldItemName)

 };
/*===========================================================================
 *		End of Class CObRefrRecord Definition
 *=========================================================================*/


#endif
/*===========================================================================
 *		End of File ObRefrRecord.H
 *=========================================================================*/
