// sampletes4.cpp : Defines the entry point for the console application.
//
#include "stdafx.h"
#include "../../tes4lib/modfile/obsimplerecordhandler.h"
#include "../../tes4lib/common/obutils.h"


/*===========================================================================
 *
 * Global Definitions
 *
 *=========================================================================*/
  CObSimpleRecordHandler	g_RecordHandler;
  dword						g_OutputCount = 0;
/*===========================================================================
 *		End of Function DisplayError()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - int DisplayError (void);
 *
 * Outputs the last error message to stdout. Returns the last set error 
 * code.
 *
 *=========================================================================*/
int DisplayError (void) {
  CObError* pLastError = ObErrorHandler->GetLastError();

	/* Special case for no error */
  if (pLastError == NULL) {
    printf ("No error occurred!\n");
    return (0);
   }

  if (pLastError->GetType() == OBERR_TYPE_GENERAL) {
    printf ("   ERROR: %s\n", pLastError->GetErrorMessage()); 
   }
  else {
    printf ("   ERROR: %s\n", pLastError->GetErrorMessage()); 
    printf ("          %s\n", pLastError->GetErrorString().c_str());
   }

  return pLastError->GetCode();
 }
/*===========================================================================
 *		End of Function DisplayError()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool OutputReference (pReference, pParentCell, pWorld);
 *
 *=========================================================================*/
bool OutputReference (CObRefrRecord* pReference, CObCellRecord* pParentCell, CObWrldRecord* pWorld) {
	CObRecord*     pRecord;
	CObStatRecord* pStatic;
	
	pRecord = g_RecordHandler.FindFormID(pReference->GetBaseFormID());
	if (pRecord == NULL) return (false);

		/* Only check static references */
	if (pRecord->GetRecordType() != OB_NAME_STAT) return (false);

	pStatic = ObCastClass(CObStatRecord, pRecord);
	if (pStatic == NULL) return (false);

	if (pWorld == NULL && pParentCell == NULL) {
		printf ("World ?, Cell ?, Reference 0x%06X, Static %s(0x%06X)\n", pReference->GetFormID(), pStatic->GetEditorID(), pStatic->GetFormID());
	}
	else if (pParentCell == NULL) {
		printf ("World %s, Cell ?, Reference 0x%06X, Static %s(0x%06X)\n", pWorld->GetEditorID(), pReference->GetFormID(), pStatic->GetEditorID(), pStatic->GetFormID());
	}
	else if (pWorld == NULL) {
		printf ("World ?, Cell %s, Reference 0x%06X, Static %s(0x%06X)\n", pParentCell->GetEditorID(), pReference->GetFormID(), pStatic->GetEditorID(), pStatic->GetFormID());
	}
	else {
		printf ("World %s, Cell %s, Reference 0x%06X, Static %s(0x%06X)\n", pWorld->GetEditorID(), pParentCell->GetEditorID(), pReference->GetFormID(), pStatic->GetEditorID(), pStatic->GetFormID());
	}

	++g_OutputCount;
	return (true);
}
/*===========================================================================
 *		End of Function OutputReference()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool OutputStaticRefs (pParentGroup, pParentCell);
 *
 *=========================================================================*/
bool OutputStaticRefs (CObGroup* pParentGroup, CObCellRecord* pParentCell, CObWrldRecord* pWorld) {
	CObBaseRecord* pBaseRecord;
	CObRefrRecord* pReference;
	CObGroup*      pGroup;
	dword          Index;
	bool           Result;

	for (Index = 0; Index < pParentGroup->GetNumRecords(); ++Index){ 
		pBaseRecord = pParentGroup->GetRecord(Index);
		pGroup = ObCastClass(CObGroup, pBaseRecord);

		if (pGroup != NULL) {
			Result = OutputStaticRefs(pGroup, pParentCell, pWorld);
			if (!Result) return (false);
		}
		else if (pBaseRecord->GetRecordType() == OB_NAME_CELL) {
			pParentCell = ObCastClass(CObCellRecord, pBaseRecord);
		}
		else if (pBaseRecord->GetRecordType() == OB_NAME_REFR) {
			pReference = ObCastClass(CObRefrRecord, pBaseRecord);
			if (pReference != NULL) OutputReference(pReference, pParentCell, pWorld);
		}
	}

	return (true);
}
/*===========================================================================
 *		End of Function OutputStaticRefs()
 *=========================================================================*/


/*===========================================================================
 *
 * Function - bool OutputAllStaticRefs (void);
 *
 *=========================================================================*/
bool OutputAllStaticRefs (void) {
	CObTypeGroup*  pWorldGroup;
	CObGroup*      pGroup;
	CObBaseRecord* pBaseRecord;
	CObWrldRecord* pWorldRecord = NULL;
	dword          Index;
	bool           Result;
	
	pWorldGroup = g_RecordHandler.GetTopGroup()->GetTypeGroup(OB_NAME_WRLD);
	if (pWorldGroup == NULL) return	AddObGeneralError("Plugin has no worldspaces!");

	for (Index = 0; Index < pWorldGroup->GetNumRecords(); ++Index){ 
			pBaseRecord = pWorldGroup->GetRecord(Index);
			pGroup      = ObCastClass(CObGroup, pBaseRecord);

			if (pGroup != NULL) {
				Result = OutputStaticRefs(pGroup, NULL, pWorldRecord);
				if (!Result) return (false);
			}
			else if (pBaseRecord->GetRecordType() == OB_NAME_WRLD) {
				pWorldRecord = ObCastClass(CObWrldRecord, pBaseRecord);
			}
	}

	return (true);
}
/*===========================================================================
 *		End of Function OutputAllStaticRefs()
 *=========================================================================*/


/*===========================================================================
 *
 * Begin Main Function
 *
 *=========================================================================*/
int _tmain(int argc, _TCHAR* argv[]) {
	CSString  Buffer;
	bool      Result;
	obtimer_t Timer;
	double    DeltaTime;

	SystemLog.Open("sampletes4.log");

		/* Get the current oblivion installation path from the registry */
	Result = GetObInstallPath(Buffer);

	if (!Result) {
		AddObGeneralError("Oblivion install path not found!");
		return DisplayError();
	}

	Buffer += "data\\oblivion.esm";
	printf ("Loading %s...\n", Buffer.c_str());
    ObStartTimer(Timer);

	Result = g_RecordHandler.Load(Buffer);
	if (!Result) return DisplayError();

	DeltaTime = ObEndTimer(Timer);
	printf("\tLoaded in %g secs...\n",DeltaTime);

	OutputAllStaticRefs();

	printf ("Found and output %u static references!\n", g_OutputCount);
	return 0;
}
/*===========================================================================
 *		End of Main Function
 *=========================================================================*/

